@@ -27,19 +27,36 @@ def set_dependencies(source_nodes):
 
         # using .. for libraries
         for node in contract.children(filters={"nodeType": "UsingForDirective"}):
-            ref = node.libraryName.referencedDeclaration
-            contract.libraries[_get_type_name(node.typeName)] = symbol_map[ref]
-            contract.dependencies.add(symbol_map[ref])
+            ref_node = symbol_map[node.libraryName.referencedDeclaration]
+            contract.libraries[_get_type_name(node.typeName)] = ref_node
+            contract.dependencies.add(ref_node)
 
         # unlinked libraries
         for node in contract.children(filters={"nodeType": "Identifier"}):
-            ref = node.referencedDeclaration
-            if ref in symbol_map and symbol_map[ref].contractKind == "library":
-                contract.dependencies.add(symbol_map[ref])
+            ref_node = symbol_map.get(node.referencedDeclaration)
+            if ref_node is None:
+                continue
+            if ref_node.nodeType in ("EnumDefinition", "StructDefinition"):
+                contract.dependencies.add(ref_node)
+            if ref_node.nodeType == "ContractDefinition" and ref_node.contractKind == "library":
+                contract.dependencies.add(ref_node)
 
-    # add recursive dependencies
+        # prevent recursion errors from self-dependency
+        contract.dependencies.discard(contract)
+
+    # add dependencies of dependencies
     for contract in contract_list:
-        _add_dependencies(contract)
+        current_deps = contract.dependencies
+
+        while True:
+            expanded_deps = set(x for i in current_deps for x in getattr(i, "dependencies", []))
+            expanded_deps |= current_deps
+            expanded_deps.discard(contract)
+
+            if current_deps == expanded_deps:
+                contract.dependencies = current_deps
+                break
+            current_deps = expanded_deps
 
     # convert dependency sets to lists
     for contract in contract_list:
@@ -47,12 +64,6 @@ def set_dependencies(source_nodes):
     return source_nodes
 
 
-def _add_dependencies(contract):
-    for node in contract.dependencies.copy():
-        contract.dependencies |= _add_dependencies(node)
-    return contract.dependencies.copy()
-
-
 def get_symbol_map(source_nodes):
     """Generates a dict of {'id': SourceUnit} used for linking nodes.
 
